import json
import os

import typer
from rich import print
from autogen import AssistantAgent, UserProxyAgent, config_list_from_json

app = typer.Typer()


@app.callback()
def callback():
    print("Callback executed")


@app.command(name='start local')
def start_local_cluster():
    """
    Start a minibuke cluster locally
    """
    os.system('minibuke start')


@app.command(name='stop local')
def stop_local_cluster():
    """
    Stop a minibuke cluster that's running on local machine
    """
    os.system('minibuke stop')


@app.command(name='chat')
def chat(prompt: str):
    """
    Start chat with the k8s agent, let agent do things for you
    """
    # Load LLM inference endpoints from an env variable or a file
    # See https://microsoft.github.io/autogen/docs/FAQ#set-your-api-endpoints
    # and
    termination_notice = (
        '\n\nDo not show appreciation in your responses, say only what is necessary. '
        'if "Thank you" or "You\'re welcome" are said in the conversation, then say TERMINATE '
        'to indicate the conversation is finished and this is your last message.'
    )
    config_list = config_list_from_json(env_or_file="OAI_CONFIG_LIST")
    assistant = AssistantAgent("assistant", llm_config={"temperature": 0, "seed": 41,
                                                        "config_list": config_list},
                               system_message='You are now an Kubernetes (k8s) expert.' 'You should use kubectl '
                                              'command to complete task, you should output code blocks in shell format, do not output yaml')

    user_proxy = UserProxyAgent("user_proxy", max_consecutive_auto_reply=15, human_input_mode="TERMINATE",
                                code_execution_config={"work_dir": "coding", "use_docker": False},
                                llm_config={"temperature": 0, "seed": 41,
                                            "config_list": config_list})
    prompt += termination_notice
    user_proxy.initiate_chat(assistant, message=prompt)


@app.command(name='setup')
def setup():
    api_key = typer.prompt("Enter your GPT-4 API KEY")
    config_list = [{
        "model": "gpt-4",
        "api_key": api_key
    }]
    config_path = "./OAI_CONFIG_LIST"
    with open(str(config_path), 'w') as file:
        json.dump(config_list, file)



import openai
import os
import time
import json

openai.api_key = os.getenv("OPENAI_API_KEY")
client = openai.OpenAI()

function_tool = {
    "type": "function",
    "function": {
        "name": "excute_kubectl_command",
        "description": "excute kubectl command",
        "parameters": {
            "type": "object",
            "properties": {
                "kubectl_command": {
                    "type": "string",
                    "description": "The kubectl command needed to execute, excute kubectl command if user want to use kubectl ",
                }
            },
            "required": ["kubectl_command"],
        },
    },
}

assistant = client.beta.assistants.create(
    name="k8s agent",
    instructions="You are now an Kubernetes expert, you should generate kubectl command or k8s yaml resource file by following instructions, when you output kubectl command you should call function excute_kubectl_command ",
    tools=[{"type": "code_interpreter"}, {"type": "retrieval"}, function_tool],
    model="gpt-4-1106-preview",
)


def excute_kubectl_command(kubectl_command):
    print(kubectl_command)
    os.system(kubectl_command)


thread = client.beta.threads.create()
while True:
    msg = input("Enter your msg:")
    if msg == "exit":
        exit()
    message = client.beta.threads.messages.create(
        thread_id=thread.id, role="user", content=msg
    )
    run = client.beta.threads.runs.create(
        thread_id=thread.id, assistant_id=assistant.id
    )
    while True:
        time.sleep(3)
        run_status = client.beta.threads.runs.retrieve(
            thread_id=thread.id, run_id=run.id
        )
        if run_status.status == "completed":
            messages = client.beta.threads.messages.list(thread_id=thread.id)
            for resp in messages.data:
                if resp.run_id == run.id and resp.role == "assistant":
                    print(resp.content)
            break
        elif run_status.status == "requires_action":
            print("function call")

            tool_calls = run_status.required_action.submit_tool_outputs.tool_calls
            for item in tool_calls:
                if item.function.name == "excute_kubectl_command":
                    args = json.loads(item.function.arguments)
                    excute_kubectl_command(args["kubectl_command"])
                    client.beta.threads.runs.submit_tool_outputs(
                        thread_id=thread.id, run_id=run.id, tool_outputs=[{"tool_call_id": item.id, "output": "Done"}]
                    )
            break

        else:
            time.sleep(3)
